{% load i18n %}

<li class="{% if urlname == "status"%}{% trans "active" %}{% endif %}">
  <a href="{% url status.views.status_index %}"><i class="glyph-grey glyph-table"></i>{% trans " Status" %}</a>
</li>


